@@ -13,7 +13,7 @@
 **	 4 Feb 93 Access registration, Search escapes bad chars TBL
 **		  PARAMETERS TO HTSEARCH AND HTLOADRELATIVE CHANGED
 **	28 May 93 WAIS gateway explicit if no WAIS library linked in.
-**
+**	   Dec 93 Bug change around, more reentrant, etc
 ** Bugs
 **	This module assumes that that the graphic object is hypertext, as it
 **	needs to select it when it has been loaded.  A superclass needs to be
@@ -44,22 +44,130 @@
 #include "HTList.h"
 #include "HText.h"	/* See bugs above */
 #include "HTAlert.h"
-
+#include "HTFWriter.h"	/* for cache stuff */
+#include "HTTee.h"
 
 /*	These flags may be set to modify the operation of this module
 */
+PUBLIC char * HTCacheDir = 0;	/* Root for cached files or 0 for no cache */
+PUBLIC char * HTSaveLocallyDir = SAVE_LOCALLY_HOME_DIR;	/* Save & exe files */
 PUBLIC char * HTClientHost = 0;	/* Name of remote login host if any */
 PUBLIC FILE * logfile = 0;	/* File to which to output one-liners */
+PUBLIC BOOL HTForceReload = NO;	/* Force reload from cache or net */
 PUBLIC BOOL HTSecure = NO;	/* Disable access for telnet users? */
+PUBLIC BOOL using_proxy = NO;	/* are we using a proxy gateway? */
+PUBLIC BOOL HTImServer = NO;	/* cern_httpd sets this */
+PUBLIC BOOL HTImProxy = NO;	/* cern_httpd as a proxy? */
 
 /*	To generate other things, play with these:
 */
 
-PUBLIC HTFormat HTOutputFormat = NULL;
-PUBLIC HTStream* HTOutputStream = NULL;	/* For non-interactive, set this */ 
+/* PUBLIC HTFormat HTOutputFormat = NULL;	use request->output_format */
+/* PUBLIC HTStream* HTOutputStream = NULL;	use request->output_stream */ 
 
 PRIVATE HTList * protocols = NULL;   /* List of registered protocol descriptors */
 
+/* 	Superclass defn */
+
+struct _HTStream {
+	HTStreamClass * isa;
+	/* ... */
+};
+
+/*	Create  a request structure
+**	---------------------------
+*/
+
+PUBLIC HTRequest * HTRequest_new NOARGS
+{
+    HTRequest * me = (HTRequest*) calloc(1, sizeof(*me));  /* zero fill */
+    if (!me) outofmem(__FILE__, "HTRequest_new()");
+    
+    me->conversions	= HTList_new();	/* No conversions registerd yet */
+    me->output_format	= WWW_PRESENT;	/* default it to present to user */
+
+    return me;
+}
+
+
+/*	Delete a request structure
+**	--------------------------
+*/
+PUBLIC void HTRequest_delete ARGS1(HTRequest *, req)
+{
+    if (req) {
+	HTFormatDelete(req->conversions);
+	HTAACleanup(req);
+	FREE(req);
+    }
+}
+
+
+PRIVATE char * method_names[(int)MAX_METHODS + 1] =
+{
+    "INVALID-METHOD",
+    "GET",
+    "HEAD",
+    "POST",
+    "PUT",
+    "DELETE",
+    "CHECKOUT",
+    "CHECKIN",
+    "SHOWMETHOD",
+    "LINK",
+    "UNLINK",
+    NULL
+};
+
+/*	Get method enum value
+**	---------------------
+*/
+PUBLIC HTMethod HTMethod_enum ARGS1(char *, name)
+{
+    if (name) {
+	int i;
+	for (i=1; i < (int)MAX_METHODS; i++)
+	    if (!strcmp(name, method_names[i]))
+		return (HTMethod)i;
+    }
+    return METHOD_INVALID;
+}
+
+
+/*	Get method name
+**	---------------
+*/
+PUBLIC char * HTMethod_name ARGS1(HTMethod, method)
+{
+    if ((int)method > (int)METHOD_INVALID  && 
+	(int)method < (int)MAX_METHODS)
+	return method_names[(int)method];
+    else
+	return method_names[(int)METHOD_INVALID];
+}
+
+
+/*	Is method in a list of method names?
+**	-----------------------------------
+*/
+PUBLIC BOOL HTMethod_inList ARGS2(HTMethod,	method,
+				  HTList *,	list)
+{
+    char * method_name = HTMethod_name(method);
+    HTList *cur = list;
+    char *item;
+
+    while (NULL != (item = (char*)HTList_nextObject(cur))) {
+	CTRACE(stderr, " %s", item);
+	if (0==strcasecomp(item, method_name))
+	    return YES;
+    }
+    return NO;	/* Not found */
+}
+
+
+
+
 
 /*	Register a Protocol				HTRegisterProtocol
 **	-------------------
@@ -111,6 +219,87 @@ GLOBALREF  HTProtocol HTWAIS;
 #endif
 
 
+
+/*							override_proxy()
+**
+**	Check the no_proxy environment variable to get the list
+**	of hosts for which proxy server is not consulted.
+**
+**	no_proxy is a comma- or space-separated list of machine
+**	or domain names, with optional :port part.  If no :port
+**	part is present, it applies to all ports on that domain.
+**
+**	Example:
+**		no_proxy="cern.ch,some.domain:8001"
+**
+*/
+PRIVATE BOOL override_proxy ARGS1(CONST char *, addr)
+{
+    CONST char * no_proxy = getenv("no_proxy");
+    char * p = NULL;
+    char * host = NULL;
+    int port = 0;
+    int h_len = 0;
+
+    if (!no_proxy || !addr || !(host = HTParse(addr, "", PARSE_HOST)))
+	return NO;
+    if (!*host) { free(host); return NO; }
+
+    if ((p = strchr(host, ':')) != NULL) {	/* Port specified */
+	*p++ = 0;			/* Chop off port */
+	port = atoi(p);
+    }
+    else {				/* Use default port */
+	char * access = HTParse(addr, "", PARSE_ACCESS);
+	if (access) {
+	    if	    (!strcmp(access,"http"))	port = 80;
+	    else if (!strcmp(access,"gopher"))	port = 70;
+	    else if (!strcmp(access,"ftp"))	port = 21;
+	    free(access);
+	}
+    }
+    if (!port) port = 80;		/* Default */
+    h_len = strlen(host);
+
+    while (*no_proxy) {
+	CONST char * end;
+	CONST char * colon = NULL;
+	int templ_port = 0;
+	int t_len;
+
+	while (*no_proxy && (WHITE(*no_proxy) || *no_proxy==','))
+	    no_proxy++;			/* Skip whitespace and separators */
+
+	end = no_proxy;
+	while (*end && !WHITE(*end) && *end != ',') {	/* Find separator */
+	    if (*end==':') colon = end;			/* Port number given */
+	    end++;
+	}
+
+	if (colon) {
+	    templ_port = atoi(colon+1);
+	    t_len = colon - no_proxy;
+	}
+	else {
+	    t_len = end - no_proxy;
+	}
+
+	if ((!templ_port || templ_port == port)  &&
+	    (t_len > 0  &&  t_len <= h_len  &&
+	     !strncmp(host + h_len - t_len, no_proxy, t_len))) {
+	    free(host);
+	    return YES;
+	}
+	if (*end) no_proxy = end+1;
+	else break;
+    }
+
+    free(host);
+    return NO;
+}
+
+
+
 /*		Find physical name and access protocol
 **		--------------------------------------
 **
@@ -124,26 +313,39 @@ GLOBALREF  HTProtocol HTWAIS;
 **			HT_OK			Success
 **
 */
-PRIVATE int get_physical ARGS2(
-	CONST char *,		addr,
-	HTParentAnchor *,	anchor)
-{
+PRIVATE int get_physical ARGS1(HTRequest *, req)
+{    
     char * access=0;	/* Name of access method */
-    char * physical = 0;
-    
+    char * addr = HTAnchor_address((HTAnchor*)req->anchor);	/* free me */
+
+    /*
+    ** This HACK is here until we have redirection implemented.
+    ** This is used when we are recursively calling HTLoad().
+    ** We then take the physical address, because currently the
+    ** virtual address is kept in a hash table so it can't be
+    ** changed -- otherwise it wouldn't be found anymore.
+    */
+    if (HTAnchor_physical(req->anchor))
+	StrAllocCopy(addr, HTAnchor_physical(req->anchor));
+
 #ifndef NO_RULES
-    physical = HTTranslate(addr);
-    if (!physical) {
-	return HT_FORBIDDEN;
+    if (HTImServer)	/* cern_httpd has already done its own translations */
+	HTAnchor_setPhysical(req->anchor, addr);
+    else {
+	char * physical = HTTranslate(addr);
+	if (!physical) {
+	    free(addr);
+	    return HT_FORBIDDEN;
+	}
+	HTAnchor_setPhysical(req->anchor, physical);
+	free(physical);			/* free our copy */
     }
-    HTAnchor_setPhysical(anchor, physical);
-    free(physical);			/* free our copy */
 #else
-    HTAnchor_setPhysical(anchor, addr);
+    HTAnchor_setPhysical(req->anchor, addr);
 #endif
 
-    access =  HTParse(HTAnchor_physical(anchor),
-    		"file:", PARSE_ACCESS);
+    access =  HTParse(HTAnchor_physical(req->anchor),
+		      "file:", PARSE_ACCESS);
 
 /*	Check whether gateway access has been set up for this
 **
@@ -151,51 +353,82 @@ PRIVATE int get_physical ARGS2(
 */
 #define USE_GATEWAYS
 #ifdef USE_GATEWAYS
-    {
-	char * gateway_parameter, *gateway;
+    if (!override_proxy(addr)) {
+	char * gateway_parameter, *gateway, *proxy;
+
 	gateway_parameter = (char *)malloc(strlen(access)+20);
 	if (gateway_parameter == NULL) outofmem(__FILE__, "HTLoad");
+
+	/* search for proxy gateways */
 	strcpy(gateway_parameter, "WWW_");
 	strcat(gateway_parameter, access);
 	strcat(gateway_parameter, "_GATEWAY");
 	gateway = (char *)getenv(gateway_parameter); /* coerce for decstation */
+
+	/* search for proxy servers */
+	strcpy(gateway_parameter, access);
+	strcat(gateway_parameter, "_proxy");
+	proxy = (char *)getenv(gateway_parameter);
+
 	free(gateway_parameter);
-	
+
+	if (TRACE && gateway)
+	    fprintf(stderr,"Gateway found: %s\n",gateway);
+	if (TRACE && proxy)
+	    fprintf(stderr,"Proxy server found: %s\n",proxy);
+
 #ifndef DIRECT_WAIS
 	if (!gateway && 0==strcmp(access, "wais")) {
 	    gateway = DEFAULT_WAIS_GATEWAY;
 	}
 #endif
-	if (gateway) {
+	/* make sure the using_proxy variable is false */
+	using_proxy = NO;
+
+	/* proxy servers have precedence over gateway servers */
+	if (proxy) {
+	    char * gatewayed=0;
+
+            StrAllocCopy(gatewayed,proxy);
+	    StrAllocCat(gatewayed,addr);
+	    using_proxy = YES;
+	    HTAnchor_setPhysical(req->anchor, gatewayed);
+	    free(gatewayed);
+	    free(access);
+
+	    access =  HTParse(HTAnchor_physical(req->anchor),
+			      "http:", PARSE_ACCESS);
+	} else if (gateway) {
 	    char * path = HTParse(addr, "",
 	    	PARSE_HOST + PARSE_PATH + PARSE_PUNCTUATION);
 		/* Chop leading / off to make host into part of path */
 	    char * gatewayed = HTParse(path+1, gateway, PARSE_ALL);
 	    free(path);
-            HTAnchor_setPhysical(anchor, gatewayed);
+            HTAnchor_setPhysical(req->anchor, gatewayed);
 	    free(gatewayed);
 	    free(access);
 	    
-    	    access =  HTParse(HTAnchor_physical(anchor),
+    	    access =  HTParse(HTAnchor_physical(req->anchor),
     		"http:", PARSE_ACCESS);
 	}
     }
 #endif
 
+    free(addr);
 
 
 /*	Search registered protocols to find suitable one
 */
     {
-	int i, n;
+	HTList *cur;
+	HTProtocol *p;
 #ifndef NO_INIT
         if (!protocols) HTAccessInit();
 #endif
-	n = HTList_count(protocols);
-	for (i=0; i<n; i++) {
-	    HTProtocol *p = HTList_objectAt(protocols, i);
+	cur = protocols;
+	while ((p = (HTProtocol*)HTList_nextObject(cur))) {
 	    if (strcmp(p->name, access)==0) {
-		HTAnchor_setProtocol(anchor, p);
+		HTAnchor_setProtocol(req->anchor, p);
 		free(access);
 		return (HT_OK);
 	    }
@@ -214,8 +447,11 @@ PRIVATE int get_physical ARGS2(
 **	anchor.  (The public routines are called with one OR the other.)
 **
 ** On entry,
-**	addr		must point to the fully qualified hypertext reference.
-**	anchor		a pareent anchor with whose address is addr
+**	request->
+**	    anchor		a parent anchor with fully qualified
+**				hypertext reference as its address set
+**	    output_format	valid
+**	    output_stream	valid on NULL
 **
 ** On exit,
 **	returns		<0		Error has occured.
@@ -224,34 +460,49 @@ PRIVATE int get_physical ARGS2(
 **					(telnet sesssion started etc)
 **
 */
-PRIVATE int HTLoad ARGS4(
-	CONST char *,		addr,
-	HTParentAnchor *,	anchor,
-	HTFormat,		format_out,
-	HTStream *,		sink)
+PUBLIC int HTLoad ARGS1(HTRequest *, request)
 {
-    HTProtocol* p;
-    int status = get_physical(addr, anchor);
+    char	*arg = NULL;
+    HTProtocol	*p;
+    int 	status;
+
+    if (request->method == METHOD_INVALID)
+	request->method = METHOD_GET;
+    status = get_physical(request);
     if (status == HT_FORBIDDEN) {
-        return HTLoadError(sink, 500, "Access forbidden by rule");
+        return HTLoadError(request, 500,
+			   "Access forbidden by rule");
     }
     if (status < 0) return status;	/* Can't resolve or forbidden */
-    
-    p = HTAnchor_protocol(anchor);
-    return (*(p->load))(HTAnchor_physical(anchor),
-    			anchor, format_out, sink);
+
+    if(!(arg = HTAnchor_physical(request->anchor)) || !*arg) 
+    	return (-1);
+
+    p = HTAnchor_protocol(request->anchor);
+    return (*(p->load))(request);
 }
 
 
 /*		Get a save stream for a document
 **		--------------------------------
 */
-PUBLIC HTStream *HTSaveStream ARGS1(HTParentAnchor *, anchor)
+PUBLIC HTStream *HTSaveStream ARGS1(HTRequest *, request)
 {
-    HTProtocol * p = HTAnchor_protocol(anchor);
+    HTProtocol * p;
+    int status;
+    request->method = METHOD_PUT;
+    status = get_physical(request);
+    if (status == HT_FORBIDDEN) {
+        HTLoadError(request, 500,
+		    "Access forbidden by rule");
+	return NULL;	/* should return error status? */
+    }
+    if (status < 0) return NULL; /* @@ error. Can't resolve or forbidden */
+    
+    p = HTAnchor_protocol(request->anchor);
     if (!p) return NULL;
     
-    return (*p->saveStream)(anchor);
+    return (*p->saveStream)(request);
     
 }
 
@@ -265,36 +516,81 @@ PUBLIC HTStream *HTSaveStream ARGS1(HTParentAnchor *, anchor)
 **	- Trace ouput and error messages
 **
 **    On Entry,
-**	  anchor	    is the node_anchor for the document
-**        full_address      The address of the document to be accessed.
+**        request->anchor	valid for of the document to be accessed.
+**	 request->childAnchor   optional anchor within doc to be selected
+**
 **        filter            if YES, treat stdin as HTML
 **
+**	  request->anchor   is the node_anchor for the document
+**	  request->output_format is valid
+**
 **    On Exit,
 **        returns    YES     Success in opening document
 **                   NO      Failure 
 **
 */
 
-PRIVATE BOOL HTLoadDocument ARGS4(
-	CONST char *,		full_address,
-	HTParentAnchor *,	anchor,
-	HTFormat,		format_out,
-	HTStream*,		sink)
+PRIVATE BOOL HTLoadDocument ARGS1(HTRequest *,		request)
 
 {
     int	        status;
     HText *	text;
-
+    char * full_address = HTAnchor_address((HTAnchor*)request->anchor);
+    
     if (TRACE) fprintf (stderr,
       "HTAccess: loading document %s\n", full_address);
 
-    if (text=(HText *)HTAnchor_document(anchor)) {	/* Already loaded */
+    request->using_cache = NULL;
+    
+    if (!request->output_format) request->output_format = WWW_PRESENT;
+
+    if (!HTForceReload && (text=(HText *)HTAnchor_document(request->anchor)))
+    {	/* Already loaded */
         if (TRACE) fprintf(stderr, "HTAccess: Document already in memory.\n");
-        HText_select(text);
+	if (request->childAnchor) {
+	    HText_selectAnchor(text, request->childAnchor);
+	} else {
+	    HText_select(text);	
+	}
+	free(full_address);
 	return YES;
     }
     
-    status = HTLoad(full_address, anchor, format_out, sink);
+    /* Check the Cache */
+    /* Caching is ONLY done if (char*) HTCacheDir is set. Henrik 09/03-94 */
+    /* Bug: for each format, we only check whether it is ok, we
+       don't check them all and chose the best */
+    if (HTCacheDir && request->anchor->cacheItems) {
+        HTList * list = request->anchor->cacheItems;
+	HTList * cur = list;
+	HTCacheItem * item;
+
+	while ((item = (HTCacheItem*)HTList_nextObject(cur))) {
+	    HTStream * s;
+	    
+	    request->using_cache = item;
+	    
+	    s = HTStreamStack(item->format, request);
+	    if (s) {		/* format was suitable */
+	        FILE * fp = fopen(item->filename, "r");
+	    	if (TRACE) fprintf(stderr, "Cache: HIT file %s for %s\n",
+				   item->filename, 
+				   full_address);
+		if (fp) {
+		    HTFileCopy(fp, s);
+		    (*s->isa->free)(s);	/* close up pipeline */
+		    fclose(fp);
+		    free(full_address);
+		    return YES;
+		} else {
+		    fprintf(stderr, "***** Can't read cache file %s !\n",
+			    item->filename);
+		} /* file open ok */
+	    } /* stream ok */
+	} /* next cache item */
+    } /* if cache available for this anchor */
+    
+    status = HTLoad(request);
 
     
 /*	Log the access if necessary
@@ -321,6 +617,7 @@ PRIVATE BOOL HTLoadDocument ARGS4(
 	    fprintf(stderr, "HTAccess: `%s' has been accessed.\n",
 	    full_address);
 	}
+	free(full_address);
 	return YES;
     }
     
@@ -330,17 +627,21 @@ PRIVATE BOOL HTLoadDocument ARGS4(
 	    "HTAccess: `%s' has been accessed, No data left.\n",
 	    full_address);
 	}
+	free(full_address);
 	return NO;
     }
     
-    if (status<0) {		      /* Failure in accessing a document */
+    /* Bug fix thanks to Lou Montulli. Henrik 10/03-94 */
+    if (status<=0) {		      /* Failure in accessing a document */
 #ifdef CURSES
         user_message("Can't access `%s'", full_address);
 #else
 	if (TRACE) fprintf(stderr, 
 		"HTAccess: Can't access `%s'\n", full_address);
 #endif
-	HTLoadError(sink, 500, "Unable to access document.");
+	/* This is done in the specific load procedures... Henrik 07/03-94 */
+	/* HTLoadError(request, 500, "Unable to access document."); */
+	free(full_address);
 	return NO;
     }
  
@@ -350,9 +651,12 @@ PRIVATE BOOL HTLoadDocument ARGS4(
     fprintf(stderr,
     "**** HTAccess: socket or file number returned by obsolete load routine!\n");
     fprintf(stderr,
-    "**** HTAccess: Internal software error. Please mail www-bug@info.cern.ch!\n");
+    "**** HTAccess: Internal software error. Please mail www-bug@info.cern.ch quoting the version number of this software and the URL: %s!\n",
+    	full_address);
+    free(full_address);
+   
     exit(-6996);
-
+    return NO;	/* For gcc :-( */
 } /* HTLoadDocument */
 
 
@@ -371,12 +675,13 @@ PRIVATE BOOL HTLoadDocument ARGS4(
 **
 */
 
-PUBLIC BOOL HTLoadAbsolute ARGS1(CONST char *,addr)
+PUBLIC BOOL HTLoadAbsolute ARGS2(CONST char *,addr, HTRequest*, request)
 {
-   return HTLoadDocument( addr,
-       		HTAnchor_parent(HTAnchor_findAddress(addr)),
-       			HTOutputFormat ? HTOutputFormat : WWW_PRESENT,
-			HTOutputStream);
+   HTAnchor * anchor = HTAnchor_findAddress(addr);
+   request->anchor = HTAnchor_parent(anchor);
+   request->childAnchor = ((HTAnchor*)request->anchor == anchor) ?
+   			NULL : (HTChildAnchor*) anchor;
+   return HTLoadDocument(request);
 }
 
 
@@ -385,7 +690,7 @@ PUBLIC BOOL HTLoadAbsolute ARGS1(CONST char *,addr)
 **
 **    On Entry,
 **        addr     The absolute address of the document to be accessed.
-**        sink     if non-NULL, send data down this stream
+**        request->output_stream     if non-NULL, send data down this stream
 **
 **    On Exit,
 **        returns    YES     Success in opening document
@@ -397,12 +702,14 @@ PUBLIC BOOL HTLoadAbsolute ARGS1(CONST char *,addr)
 PUBLIC BOOL HTLoadToStream ARGS3(
 		CONST char *,	addr,
 		BOOL, 		filter,
-		HTStream *, 	sink)
+		HTRequest*,	request)
 {
-   return HTLoadDocument(addr,
-       		HTAnchor_parent(HTAnchor_findAddress(addr)),
-       			HTOutputFormat ? HTOutputFormat : WWW_PRESENT,
-			sink);
+   HTAnchor * anchor = HTAnchor_findAddress(addr);
+   request->anchor = HTAnchor_parent(anchor);
+   request->childAnchor = ((HTAnchor*)request->anchor == anchor) ? NULL :
+   	(HTChildAnchor*) anchor;
+    request->output_stream = request->output_stream;
+    return HTLoadDocument(request);
 }
 
 
@@ -422,9 +729,10 @@ PUBLIC BOOL HTLoadToStream ARGS3(
 **
 */
 
-PUBLIC BOOL HTLoadRelative ARGS2(
+PUBLIC BOOL HTLoadRelative ARGS3(
 		CONST char *,		relative_name,
-		HTParentAnchor *,	here)
+		HTParentAnchor *,	here,
+		HTRequest *,		request)
 {
     char * 		full_address = 0;
     BOOL       		result;
@@ -439,7 +747,7 @@ PUBLIC BOOL HTLoadRelative ARGS2(
     full_address = HTParse(stripped,
 	           current_address,
 		   PARSE_ACCESS|PARSE_HOST|PARSE_PATH|PARSE_PUNCTUATION);
-    result = HTLoadAbsolute(full_address);
+    result = HTLoadAbsolute(full_address, request);
     free(full_address);
     free(current_address);
     free(mycopy);  /* Memory leak fixed 10/7/92 -- JFG */
@@ -459,37 +767,15 @@ PUBLIC BOOL HTLoadRelative ARGS2(
 **
 */
 
-PUBLIC BOOL HTLoadAnchor ARGS1(HTAnchor *,destination)
+PUBLIC BOOL HTLoadAnchor ARGS2(HTAnchor*, anchor, HTRequest *, request)
 {
-    HTParentAnchor * parent;
-    BOOL loaded = NO;
-    if (!destination) return NO;	/* No link */
+    if (!anchor) return NO;	/* No link */
     
-    parent  = HTAnchor_parent(destination);
+    request->anchor  = HTAnchor_parent(anchor);
+    request->childAnchor = ((HTAnchor*)request->anchor == anchor) ? NULL
+    					: (HTChildAnchor*) anchor;
     
-    if (HTAnchor_document(parent) == NULL) {	/* If not alread loaded */
-    						/* TBL 921202 */
-
-        BOOL result;
-        char * address = HTAnchor_address((HTAnchor*) parent);
-	result = HTLoadDocument(address, parent,
-		HTOutputFormat ? HTOutputFormat : WWW_PRESENT,
-		HTOutputStream);
-	free(address);
-	if (!result) return NO;
-	loaded = YES;
-    }
-    
-    {
-	HText *text = (HText*)HTAnchor_document(parent);
-	if (destination != (HTAnchor *)parent) {  /* If child anchor */
-	    HText_selectAnchor(text, 
-		    (HTChildAnchor*)destination); /* Double display? @@ */
-	} else {
-	    if (!loaded) HText_select(text);
-	}
-    }
-    return YES;
+    return HTLoadDocument(request) ? YES : NO;
 	
 } /* HTLoadAnchor */
 
@@ -511,9 +797,10 @@ PRIVATE char hex(i)
     return hexchars[i];
 }
 
-PUBLIC BOOL HTSearch ARGS2(
+PUBLIC BOOL HTSearch ARGS3(
 	CONST char *, 		keywords,
-	HTParentAnchor *, 	here)
+	HTParentAnchor *, 	here,
+	HTRequest *,		request)
 {
 
 #define acceptable \
@@ -525,8 +812,9 @@ PUBLIC BOOL HTSearch ARGS2(
     BOOL result;
     char * escaped = malloc(strlen(keywords)*3+1);
 
-    static CONST BOOL isAcceptable[96] =
-
+    /* static CONST BOOL isAcceptable[96] = */
+    /* static AND const is not good for a gnu compiler! Frystyk 25/02-94 */
+    static BOOL isAcceptable[96] =
     /*   0 1 2 3 4 5 6 7 8 9 A B C D E F */
     {    0,0,0,0,0,0,0,0,0,0,1,0,0,1,1,0,	/* 2x   !"#$%&'()*+,-./	 */
          1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,	/* 3x  0123456789:;<=>?	 */
@@ -537,16 +825,15 @@ PUBLIC BOOL HTSearch ARGS2(
 
     if (escaped == NULL) outofmem(__FILE__, "HTSearch");
     
+/* Convert spaces to + and hex escape unacceptable characters */
 
-/*	Convert spaces to + and hex escape unacceptable characters
-*/
-    for(s=keywords; *s && WHITE(*s); s++) /*scan */ ;	/* Skip white space */
-    for(e = s + strlen(s); e>s && WHITE(*(e-1)) ; e--); /* Skip trailers */
-    for(q=escaped, p=s; p<e; p++) {			/* scan stripped field */
+    for(s=keywords; *s && WHITE(*s); s++); /*scan */ 	 /* Skip white space */
+    for(e = s + strlen(s); e>s && WHITE(*(e-1)) ; e--);     /* Skip trailers */
+    for(q=escaped, p=s; p<e; p++) {		      /* scan stripped field */
         int c = (int)TOASCII(*p);
         if (WHITE(*p)) {
 	    *q++ = '+';
-	} else if (c>=32 && c<=(char)127 && isAcceptable[c-32]) {
+	} else if (c>=32 && c<=127 && isAcceptable[c-32] != 0) {
 	    *q++ = *p;			/* 930706 TBL for MVS bug */
 	} else {
 	    *q++ = '%';
@@ -563,7 +850,7 @@ PUBLIC BOOL HTSearch ARGS2(
     StrAllocCat(address, "?");
     StrAllocCat(address, escaped);
     free(escaped);
-    result = HTLoadRelative(address, here);
+    result = HTLoadRelative(address, here, request);
     free(address);
     
     return result;
@@ -580,13 +867,14 @@ PUBLIC BOOL HTSearch ARGS2(
 **	*addres		is name of object search is to be done on.
 */
 
-PUBLIC BOOL HTSearchAbsolute ARGS2(
+PUBLIC BOOL HTSearchAbsolute ARGS3(
 	CONST char *, 	keywords,
-	CONST char *, 	indexname)
+	CONST char *, 	indexname,
+	HTRequest *,	request)
 {
     HTParentAnchor * anchor =
     	(HTParentAnchor*) HTAnchor_findAddress(indexname);
-    return HTSearch(keywords, anchor);
+    return HTSearch(keywords, anchor, request);
 }
 
 
@@ -681,3 +969,28 @@ PUBLIC HTParentAnchor * HTHomeAnchor NOARGS
 }
 
 
+/*		Bind an Anchor to the request structure
+**		---------------------------------------
+**
+**    On Entry,
+**	anchor		The child or parenet anchor to be binded
+**	request		The request sturcture
+**    On Exit,
+**        returns    YES     Success
+**                   NO      Failure 
+**
+**  Note: Actually the same as HTLoadAnchor() but DOES NOT do the loading
+**						Henrik Frystyk 17/02-94
+*/
+
+PUBLIC BOOL HTBindAnchor ARGS2(HTAnchor*, anchor, HTRequest *, request)
+{
+    if (!anchor) return NO;	/* No link */
+    
+    request->anchor  = HTAnchor_parent(anchor);
+    request->childAnchor = ((HTAnchor*)request->anchor == anchor) ? NULL
+    					: (HTChildAnchor*) anchor;
+	
+    return YES;
+} /* HTBindAnchor */
+
