@@ -51,6 +51,7 @@ BUGS:	@@@  	Limit connection cache size!
 **		same time.
 */		
 
+#include "HTFormat.h"
 #include "HTFTP.h"	/* Implemented here */
 
 #define CR   FROMASCII('\015')	/* Must be converted to ^M for transmission */
@@ -91,6 +92,7 @@ typedef struct _connection {
     u_long			addr;	/* IP address		*/
     int				socket;	/* Socket number for communication */
     BOOL			binary; /* Binary mode? */
+    HTInputSocket *		isoc;
 } connection;
 
 #ifndef NIL
@@ -134,8 +136,6 @@ PRIVATE	unsigned short	passive_port;	/* Port server specified for data */
 #endif
 
 
-#define NEXT_CHAR HTGetChararcter()	/* Use function in HTFormat.c */
-
 #define DATA_BUFFER_SIZE 2048
 PRIVATE char data_buffer[DATA_BUFFER_SIZE];		/* Input data buffer */
 PRIVATE char * data_read_pointer;
@@ -180,6 +180,11 @@ PRIVATE int close_connection(con)
 {
     connection * scan;
     int status = NETCLOSE(con->socket);
+    
+    if(con->isoc) {
+	HTInputSocket_free(con->isoc);
+	con->isoc = NULL;
+    }
     if (TRACE) fprintf(stderr, "FTP: Closing control socket %d\n", con->socket);
     if (connections==con) {
         connections = con->next;
@@ -217,12 +222,8 @@ PRIVATE int close_connection(con)
 **	returns:  The first digit of the reply type,
 **		  or negative for communication failure.
 */
-#ifdef __STDC__
-PRIVATE int response(char * cmd)
-#else
-PRIVATE int response(cmd)
-    char * cmd;
-#endif
+PRIVATE int response ARGS1(char *, cmd)
+
 {
     int result;				/* Three-digit decimal code */
     int	continuation_response = -1;
@@ -258,7 +259,7 @@ PRIVATE int response(cmd)
     do {
 	char *p = response_text;
 	for(;;) {  
-	    if (((*p++=NEXT_CHAR) == LF)
+	    if (((*p++=HTInputSocket_getCharacter(control->isoc)) == LF)
 			|| (p == &response_text[LINE_LENGTH])) {
 		char continuation;
 		*p++=0;			/* Terminate the string */
@@ -382,7 +383,7 @@ PRIVATE int get_connection ARGS1 (CONST char *,arg)
 */      
     {
         int status;
-	connection * con = (connection *)malloc(sizeof(*con));
+	connection * con = (connection *)calloc(1, sizeof(*con));
 	if (con == NULL) outofmem(__FILE__, "get_connection");
 	con->addr = sin->sin_addr.s_addr;	/* save it */
 	con->binary = NO;
@@ -412,7 +413,7 @@ PRIVATE int get_connection ARGS1 (CONST char *,arg)
 	control = con;			/* Current control connection */
 	con->next = connections;	/* Link onto list of good ones */
 	connections = con;
-        HTInitInput(con->socket);/* Initialise buffering for contron connection */
+        con->isoc = HTInputSocket_new(con->socket); /* buffering */
 
 
 /*	Now we log in		Look up username, prompt for pw.
@@ -486,7 +487,7 @@ PRIVATE int get_connection ARGS1 (CONST char *,arg)
 #endif
 	return con->socket;			/* Good return */
     } /* Scope of con */
-}
+}  /* get_connection */
 
 
 #ifdef LISTEN
@@ -655,30 +656,33 @@ PRIVATE int get_listen_socket()
 **	returns		HT_LOADED if OK
 **			<0 if error.
 */
-PRIVATE int read_directory
-ARGS4 (
+PRIVATE int read_directory ARGS1(HTRequest *, request)
+#if OLD_PARAMS
+ARGS6 (
+  HTRequest *,			request,
+  void *,			param,
   HTParentAnchor *,		parent,
   CONST char *,			address,
   HTFormat,			format_out,
   HTStream *,			sink )
+#endif
 {
-    HTStructured* target = HTML_new(parent, format_out, sink);
+    HTStructured* target = HTML_new(request, NULL, WWW_HTML, request->output_format, request->output_stream);
     HTStructuredClass targetClass;
+    char *address = HTAnchor_physical(request->anchor);
     char *filename = HTParse(address, "", PARSE_PATH + PARSE_PUNCTUATION);
-
-    char c = 0;
-
     char *lastpath;  /* prefix for link, either "" (for root) or xxx  */
-    char *entry;   /* pointer into lastpath to bit after last slash */
 
     targetClass = *(target->isa);
 
-    HTDirTitles(target, (HTAnchor*)parent);
+    HTDirTitles(target, (HTAnchor*)request->anchor);
   
     data_read_pointer = data_write_pointer = data_buffer;
 
-    if (*filename == 0)  /* Empty filename : use root */
+    if (*filename == 0) { /* Empty filename : use root */
+	lastpath = (char*)malloc(2);
         strcpy (lastpath, "/");
+    }
     else 
     {
         char * p = strrchr(filename, '/');  /* find lastslash */
@@ -697,7 +701,6 @@ ARGS4 (
 	for (c=0; c!=(char)EOF;)   /* For each entry in the directory */
 	{
 	    char * filename = NULL;
-	    char * p = entry;
 	    HTChunkClear(chunk);
 	    /*   read directory entry
 	     */
@@ -753,14 +756,19 @@ ARGS4 (
 **	returns		Socket number for file if good.
 **			<0 if bad.
 */
-PUBLIC int HTFTPLoad
-ARGS4 (
+PUBLIC int HTFTPLoad ARGS1(HTRequest *, request)
+#ifdef OLD_PARAMS
+ARGS6 (
+  HTRequest *,			request,
+  void *,			param,
   CONST char *,			name,
   HTParentAnchor *,		anchor,
   HTFormat,			format_out,
   HTStream *,			sink
 )
+#endif
 {
+    char * name = HTAnchor_physical(request->anchor);
     BOOL isDirectory = NO;
     int status;
     int retry;			/* How many times tried? */
@@ -855,12 +863,16 @@ ARGS4 (
     {
         char *filename = HTParse(name, "", PARSE_PATH + PARSE_PUNCTUATION);
 	char command[LINE_LENGTH+1];
-	BOOL binary;
-	HTAtom * encoding;
+	BOOL binary = NO;
+
 	if (!*filename) StrAllocCopy(filename, "/");
-	format = HTFileFormat(filename, &encoding);
-	binary = (encoding != HTAtom_for("8bit")
-		  && encoding != HTAtom_for("7bit"));
+	HTUnEscape(filename);	/* Fix 25.1.94 -- thanks to Lou Montulli */
+	if (TRACE) fprintf(stderr, "FTP: UnEscaped %s\n", filename);
+	format = HTFileFormat(filename,
+			      &request->content_encoding,
+			      &request->content_language);
+	binary = (request->content_encoding != HTAtom_for("8bit")
+		  && request->content_encoding != HTAtom_for("7bit"));
         if (binary != control->binary) {
 	    char * mode = binary ? "I" : "A";
 	    sprintf(command, "TYPE %s%c%c", mode, CR, LF);
@@ -898,17 +910,13 @@ ARGS4 (
 	data_soc = status;
     }
 #else
-/* @@ */
+    /* @@ */
 #endif
     if (isDirectory) {
-	return read_directory (anchor, name, format_out, sink);
+	return read_directory(request);
       /* returns HT_LOADED or error */
     } else {
-	HTParseSocket(format, format_out,
-		anchor, data_soc, sink);
-		
-	HTInitInput(control->socket);
-	/* Reset buffering to control connection DD 921208 */
+	HTParseSocket(format, data_soc, request);
     
 	status = NETCLOSE(data_soc);
 	if (TRACE) fprintf(stderr, "FTP: Closing data socket %d\n", data_soc);
@@ -916,7 +924,7 @@ ARGS4 (
 	data_soc = -1;	/* invalidate it */
 	
 	status = response(NIL);		/* Pick up final reply */
-	if (status!=2) return HTLoadError(sink, 500, response_text);
+	if (status!=2) return HTLoadError(request, 500, response_text);
 
 	return HT_LOADED;
     }       
