@@ -9,7 +9,7 @@
 **	Override this module if making a new GUI browser.
 **
 */
-#include "HTMLDTD.h"
+
 #include "HTML.h"
 #include <ctype.h>
 #include <stdio.h>
@@ -23,6 +23,18 @@
 
 #include "TkWWWCmds.h"
 
+/* These defines should be in HTMLPDTD.h but aren't */
+#define HTML_BASE_HREF 0
+
+#define HTML_LINK_HREF 0
+#define HTML_LINK_IDREF 1
+#define HTML_LINK_METHODS 2
+#define HTML_LINK_REL 3
+#define HTML_LINK_REV 4
+
+
+PUBLIC HTStyleSheet *styleSheet = NULL;
+
 /*		HTML Object
 **		-----------
 */
@@ -40,6 +52,8 @@ struct _HTStructured {
     char *			comment_start;	/* for literate programming */
     char *			comment_end;
 
+    CONST SGML_dtd*		dtd;
+
     BOOL                        in_title;
     BOOL                        in_comment;
     BOOL			in_word;  /* Have just had a non-white char */
@@ -53,101 +67,78 @@ struct _HTStream {
     /* .... */
 };
 
+static char * ISO_Latin1[] = {
+        "\306", /* capital AE diphthong (ligature) */ 
+        "\301", /* capital A, acute accent */ 
+        "\302", /* capital A, circumflex accent */ 
+        "\300", /* capital A, grave accent */ 
+        "\305", /* capital A, ring */ 
+        "\303", /* capital A, tilde */ 
+        "\304", /* capital A, dieresis or umlaut mark */ 
+        "\307", /* capital C, cedilla */ 
+        "\320", /* capital Eth, Icelandic */ 
+        "\311", /* capital E, acute accent */ 
+        "\312", /* capital E, circumflex accent */ 
+        "\310", /* capital E, grave accent */ 
+        "\313", /* capital E, dieresis or umlaut mark */ 
+        "\315", /* capital I, acute accent */ 
+        "\316", /* capital I, circumflex accent */ 
+        "\314", /* capital I, grave accent */ 
+        "\317", /* capital I, dieresis or umlaut mark */ 
+        "\321", /* capital N, tilde */ 
+        "\323", /* capital O, acute accent */ 
+        "\324", /* capital O, circumflex accent */ 
+        "\322", /* capital O, grave accent */ 
+        "\330", /* capital O, slash */ 
+        "\325", /* capital O, tilde */ 
+        "\326", /* capital O, dieresis or umlaut mark */ 
+        "\336", /* capital THORN, Icelandic */ 
+        "\332", /* capital U, acute accent */ 
+        "\333", /* capital U, circumflex accent */ 
+        "\331", /* capital U, grave accent */ 
+        "\334", /* capital U, dieresis or umlaut mark */ 
+        "\335", /* capital Y, acute accent */ 
+        "\341", /* small a, acute accent */ 
+        "\342", /* small a, circumflex accent */ 
+        "\346", /* small ae diphthong (ligature) */ 
+        "\340", /* small a, grave accent */ 
+        "\046", /* ampersand */ 
+        "\345", /* small a, ring */ 
+        "\343", /* small a, tilde */ 
+        "\344", /* small a, dieresis or umlaut mark */ 
+        "\347", /* small c, cedilla */ 
+        "\351", /* small e, acute accent */ 
+        "\352", /* small e, circumflex accent */ 
+        "\350", /* small e, grave accent */ 
+        "\360", /* small eth, Icelandic */ 
+        "\353", /* small e, dieresis or umlaut mark */ 
+        "\076", /* greater than */ 
+        "\355", /* small i, acute accent */ 
+        "\356", /* small i, circumflex accent */ 
+        "\354", /* small i, grave accent */ 
+        "\357", /* small i, dieresis or umlaut mark */ 
+        "\074", /* less than */ 
+        "\361", /* small n, tilde */ 
+        "\363", /* small o, acute accent */ 
+        "\364", /* small o, circumflex accent */ 
+        "\362", /* small o, grave accent */ 
+        "\370", /* small o, slash */ 
+        "\365", /* small o, tilde */ 
+        "\366", /* small o, dieresis or umlaut mark */ 
+        "\337", /* small sharp s, German (sz ligature) */ 
+        "\376", /* small thorn, Icelandic */ 
+        "\372", /* small u, acute accent */ 
+        "\373", /* small u, circumflex accent */ 
+        "\371", /* small u, grave accent */ 
+        "\374", /* small u, dieresis or umlaut mark */ 
+        "\375", /* small y, acute accent */ 
+        "\377", /* small y, dieresis or umlaut mark */ 
+      };
 
 /* 	Entity values -- for ISO Latin 1 local representation
 **
 **	This MUST match exactly the table referred to in the DTD!
 */
-static char * ISO_Latin1[] = {
-  	"\306",	/* capital AE diphthong (ligature) */ 
-  	"\301",	/* capital A, acute accent */ 
-  	"\302",	/* capital A, circumflex accent */ 
-  	"\300",	/* capital A, grave accent */ 
-  	"\305",	/* capital A, ring */ 
-  	"\303",	/* capital A, tilde */ 
-  	"\304",	/* capital A, dieresis or umlaut mark */ 
-  	"\307",	/* capital C, cedilla */ 
-  	"\320",	/* capital Eth, Icelandic */ 
-  	"\311",	/* capital E, acute accent */ 
-  	"\312",	/* capital E, circumflex accent */ 
-  	"\310",	/* capital E, grave accent */ 
-  	"\313",	/* capital E, dieresis or umlaut mark */ 
-  	"\315",	/* capital I, acute accent */ 
-  	"\316",	/* capital I, circumflex accent */ 
-  	"\314",	/* capital I, grave accent */ 
-  	"\317",	/* capital I, dieresis or umlaut mark */ 
-  	"\321",	/* capital N, tilde */ 
-  	"\323",	/* capital O, acute accent */ 
-  	"\324",	/* capital O, circumflex accent */ 
-  	"\322",	/* capital O, grave accent */ 
-  	"\330",	/* capital O, slash */ 
-  	"\325",	/* capital O, tilde */ 
-  	"\326",	/* capital O, dieresis or umlaut mark */ 
-  	"\336",	/* capital THORN, Icelandic */ 
-  	"\332",	/* capital U, acute accent */ 
-  	"\333",	/* capital U, circumflex accent */ 
-  	"\331",	/* capital U, grave accent */ 
-  	"\334",	/* capital U, dieresis or umlaut mark */ 
-  	"\335",	/* capital Y, acute accent */ 
-  	"\341",	/* small a, acute accent */ 
-  	"\342",	/* small a, circumflex accent */ 
-  	"\346",	/* small ae diphthong (ligature) */ 
-  	"\340",	/* small a, grave accent */ 
-  	"\046",	/* ampersand */ 
-  	"\345",	/* small a, ring */ 
-  	"\343",	/* small a, tilde */ 
-  	"\344",	/* small a, dieresis or umlaut mark */
-        "\246", /* broken vertical bar */ 
-  	"\347",	/* small c, cedilla */ 
-  	"\242",	/* cent sign */
-        "\251", /* copyright sign */
-	"\260", /* degree sign */
-  	"\351",	/* small e, acute accent */ 
-  	"\352",	/* small e, circumflex accent */ 
-  	"\350",	/* small e, grave accent */ 
-  	"\360",	/* small eth, Icelandic */ 
-  	"\353",	/* small e, dieresis or umlaut mark */
-	"\275", /* fraction 1/4 */
-	"\276", /* fraction 1/2 */
-	"\277", /* fraction 3/4 */
-  	"\076",	/* greater than */ 
-  	"\355",	/* small i, acute accent */ 
-  	"\356",	/* small i, circumflex accent */ 
-  	"\241",	/* inverted exclamation mark */
-  	"\354",	/* small i, grave accent */
-  	"\277",  /* inverted question mark */ 
-  	"\357",	/* small i, dieresis or umlaut mark */ 
-	"\254", /* angle quotation mark, left */
-  	"\074",	/* less than */ 
-  	"\181",	/* micro sign */
-  	"\267",	/* center dot */
-  	"\254",	/* negation sign */
-  	"\361",	/* small n, tilde */ 
-  	"\363",	/* small o, acute accent */ 
-  	"\364",	/* small o, circumflex accent */ 
-  	"\362",	/* small o, grave accent */ 
-  	"\370",	/* small o, slash */ 
-  	"\365",	/* small o, tilde */ 
-  	"\366",	/* small o, dieresis or umlaut mark */
-  	"\266",	/* paragraph sign */
-  	"\261",	/* plus or minus sign */  
-        "\243", /* pound sign */
-  	"\274",	/* angle quotation mark, right */
-  	"\256",	/* circled R registered sign */
-        "\247", /* section sign */
-  	"\271",	/* superscript 1 */
-  	"\262",	/* superscript 2 */
-        "\263",	/* superscript 3 */
-  	"\337",	/* small sharp s, German (sz ligature) */ 
-  	"\376",	/* small thorn, Icelandic */ 
-  	"\372",	/* small u, acute accent */ 
-  	"\373",	/* small u, circumflex accent */ 
-  	"\371",	/* small u, grave accent */ 
-  	"\374",	/* small u, dieresis or umlaut mark */ 
-  	"\375",	/* small y, acute accent */ 
-        "\245", /* yen sign */
-  	"\377",	/* small y, dieresis or umlaut mark */ 
-};
 
 /*		Set character set
 **		----------------
@@ -312,7 +303,7 @@ PRIVATE void addCommand ARGS3(HTStructured *, me, CONST char *, command,
   begin_command(me);
   HText_puts(me->text, command);
   HText_putc(me->text, ' ');
-  HText_puts(me->text, HTML_dtd.tags[elem].name);  
+  HText_puts(me->text, me->dtd->tags[elem].name);  
   HText_puts(me->text, "\n");
   HText_executeTCL(me->text);
 }
@@ -355,10 +346,10 @@ PRIVATE void HTML_start_element ARGS4(
 	begin_command(me);
 	HText_puts(me->text, "tkW3HtSetLink \"");
 	HText_puts(me->text, 
-		   present[HTML_A_REL] ? value[HTML_A_REL] : "");
+		   present[HTML_LINK_REL] ? value[HTML_LINK_REL] : "");
 	HText_puts(me->text, "\" \"");
 	HText_puts(me->text,
-		   present[HTML_A_HREF] ? value[HTML_A_HREF] : "");
+		   present[HTML_LINK_HREF] ? value[HTML_LINK_HREF] : "");
 	HText_puts(me->text, "\"\n");
 	HText_executeTCL(me->text);
 	break;
@@ -391,22 +382,18 @@ PRIVATE void HTML_start_element ARGS4(
 	  HText_puts(me->text, "\"\n");
 	}
 	break;
-	
+
       case HTML_ISINDEX:
    	HTAnchor_setIndex(me->node_anchor);
 	begin_command(me);
 	HText_puts(me->text, "tkW3HtSetInd \"");
 	if (present[HTML_BASE_HREF])
-	  HText_puts(me->text, value[HTML_BASE_HREF]);
+	  HText_puts(me->text, value[HTML_BASE_HREF]); 
 	HText_puts(me->text, "\"\n");
 	HText_executeTCL(me->text);
 	break;
 	
       case HTML_DL:
-	beginListStyle(me, present && present[HTML_DL_COMPACT]
-		       ? HTML_DLC : HTML_DL);
-	break;
-
       case HTML_UL:
       case HTML_OL:
       case HTML_MENU:
@@ -419,6 +406,7 @@ PRIVATE void HTML_start_element ARGS4(
       case HTML_LI:
       case HTML_P:
       case HTML_BR:
+      case HTML_HR:
 	appendBullet(me, element_number);
 	me->in_word = NO;
 	break;
@@ -642,28 +630,31 @@ PUBLIC CONST HTStructuredClass HTMLPresentation = /* As opposed to print etc */
 **	The strutcured stream can generate either presentation,
 **	or plain text, or HTML.
 */
-PUBLIC HTStructured* HTML_new ARGS3(
-	HTParentAnchor *, 	anchor,
-	HTFormat,		format_out,
-	HTStream*,		stream)
+PUBLIC HTStructured* HTML_new ARGS5(
+	HTRequest *,		request,
+	void *,			param,
+	HTFormat,		input_format,
+	HTFormat,		output_format,
+	HTStream *,		output_stream)
 {
 
     HTStructured * me;
     
-    if (format_out != WWW_PLAINTEXT && format_out != WWW_PRESENT) {
-        HTStream * intermediate = HTStreamStack(WWW_HTML, format_out,
-		stream, anchor);
-	if (intermediate) return HTMLGenerator(intermediate);
-        fprintf(stderr, "** Internal error: can't parse HTML to %s\n",
-       		HTAtom_name(format_out));
-	exit (-99);
-    }
-
     me = (HTStructured*) malloc(sizeof(*me));
     if (me == NULL) outofmem(__FILE__, "HTML_new");
-
+    if (output_format != WWW_PLAINTEXT
+    	&& output_format != WWW_PRESENT
+	&& output_format != HTAtom_for("text/x-c")) {
+      HTStream * intermediate = HTStreamStack(WWW_HTML, request);
+      if (intermediate) return HTMLGenerator(intermediate);
+      fprintf(stderr, "** Internal error: can't parse HTML to %s\n",
+	      HTAtom_name(output_format));
+      exit (-99);
+    }
+    
     me->isa = &HTMLPresentation;
-    me->node_anchor =  anchor;
+    me->dtd = &DTD;
+    me->node_anchor =  request->anchor;
     me->title.size = 0;
     me->title.growby = 128;
     me->title.allocated = 0;
@@ -672,31 +663,33 @@ PUBLIC HTStructured* HTML_new ARGS3(
     
     me->comment_start = NULL;
     me->comment_end = NULL;
-    me->target = stream;
+    me->target = output_stream;
     me->in_verbatim = NO;
     me->in_word = NO;
     me->in_comment = NO;
     me->in_title = NO;
     me->in_string = NO;
-    if (stream) me->targetClass = *stream->isa;	/* Copy pointers */
-    me->text = HText_new2(anchor, stream);
+    if (output_stream) me->targetClass = *output_stream->isa;	/* Copy pointers */
+    me->text = HText_new2(request->anchor, output_stream);
     HText_puts(me->text, "tkW3HtBeginDoc\n");
     HText_executeTCL(me->text);
     return (HTStructured*) me;
 }
 
-
 /*	HTConverter for HTML to plain text
 **	----------------------------------
 **
 **	This will convert from HTML to presentation or plain text.
 */
-PUBLIC HTStream* HTMLToPlain ARGS3(
-	HTPresentation *,	pres,
-	HTParentAnchor *,	anchor,	
-	HTStream *,		sink)
+PUBLIC HTStream* HTMLToPlain ARGS5(
+	HTRequest *,		request,
+	void *,			param,
+	HTFormat,		input_format,
+	HTFormat,		output_format,
+	HTStream *,		output_stream)
 {
-    return SGML_new(&HTML_dtd, HTML_new(anchor, pres->rep_out, sink));
+    return SGML_new(&DTD, HTML_new(
+    	request, NULL, input_format, output_format, output_stream));
 }
 
 
@@ -707,20 +700,23 @@ PUBLIC HTStream* HTMLToPlain ARGS3(
 **	is commented out.
 **	This will convert from HTML to presentation or plain text.
 */
-PUBLIC HTStream* HTMLToC ARGS3(
-	HTPresentation *,	pres,
-	HTParentAnchor *,	anchor,	
-	HTStream *,		sink)
+PUBLIC HTStream* HTMLToC ARGS5(
+	HTRequest *,		request,
+	void *,			param,
+	HTFormat,		input_format,
+	HTFormat,		output_format,
+	HTStream *,		output_stream)
 {
     
     HTStructured * html;
     
-    (*sink->isa->put_string)(sink, "/* ");	/* Before even title */
-    html = HTML_new(anchor, WWW_PLAINTEXT, sink);
+    (*output_stream->isa->put_string)(output_stream, "/* "); /* Before even title */
+    html = HTML_new(request, NULL, input_format, output_format, output_stream);
     html->comment_start = "/* ";
+    html->dtd = &DTD;
     html->comment_end = " */\n";	/* Must start in col 1 for cpp */
 /*    HTML_put_string(html,html->comment_start); */
-    return SGML_new(&HTML_dtd, html);
+    return SGML_new(&DTD, html);
 }
 
 
@@ -732,12 +728,15 @@ PUBLIC HTStream* HTMLToC ARGS3(
 **	Override this if you have a windows version
 */
 #ifndef GUI
-PUBLIC HTStream* HTMLPresent ARGS3(
-	HTPresentation *,	pres,
-	HTParentAnchor *,	anchor,	
-	HTStream *,		sink)
+PUBLIC HTStream* HTMLPresent ARGS5(
+	HTRequest *,		request,
+	void *,			param,
+	HTFormat,		input_format,
+	HTFormat,		output_format,
+	HTStream *,		output_stream)
 {
-    return SGML_new(&HTML_dtd, HTML_new(anchor, WWW_PRESENT, NULL));
+    return SGML_new(&DTD, HTML_new(
+    	request, NULL, input_format, output_format, output_stream));
 }
 #endif
 
@@ -764,10 +763,13 @@ PUBLIC HTStream* HTMLPresent ARGS3(
 
 
 PUBLIC int HTLoadError ARGS3(
-	HTStream *, 	sink,
+	HTRequest *, 	req,
 	int,		number,
 	CONST char *,	message)
 {
-  Tcl_SetResult(HtTclInterp, (char *) message, TCL_DYNAMIC);
+  char *err = "Oh I screwed up!";   
+  Tcl_SetResult(HtTclInterp, (char *) message, TCL_VOLATILE);
+  if (req && req->output_stream)
+    (*req->output_stream->isa->abort)(req->output_stream, err);
   return -number;
 } 
