@@ -1,8 +1,8 @@
 ## edit.tcl main file for tkWWW  user interface
 ## ==============
-## Copyright (C) 1992-1993
+## Copyright (C) 1992-1994
 ## Globewide Network Academy
-## Macvicar Institute for Educational Software Development
+## Macvicar School of Education and Technology
 ##
 ## See the file COPYRIGHT for conditions
 
@@ -51,6 +51,7 @@ proc tkW3EditInitialize {} {
 	{VAR variable}
 	{DFN definition}
 	{CITE citation}
+        {CODE code}
     }
 
     foreach item $convert_list {
