@@ -60,7 +60,13 @@ global _edtkHOME			;# user's home directory
 
 global _edtkPREFS			;# user preferences
 
-set _edtkNAME $env(USER)
+if [info exists env(USER)] {
+    set _edtkNAME $env(USER)
+} elseif [info exists env(LOGNAME)] {
+    set _edtkNAME $env(LOGNAME)
+} else {
+    error "Can't find environment variable USER or LOGNAME"
+}
 set _edtkHOME $env(HOME)
 
 # check for $HOME/.tk/jlibrary.tcl and read it in if it exists.
