@@ -30,6 +30,14 @@ set tkW3HtPage(is_index) 0
 
 set tkW3ConfigHelpRoot http://tk-www.mit.edu:8001/tk-www/help
 
+set tkW3ConfigMailList(request) vn@vungtau.cerf.net
+set tkW3ConfigMailList(list) tkwww@vungtau.cerf.net
+set tkW3ConfigMailList(bugs) tk-www-bugs@mit.edu
+
+# set the logo
+# This should be defined in the tkWWW server
+set tkW3ConfigLogo HtLogoBitmap
+
 ## The programs that tkWWW uses
 set tkW3ConfigViewer(editor) 		emacs
 set tkW3ConfigViewer(terminal)		xterm
@@ -42,15 +50,21 @@ set tkW3ConfigViewer(postscript)	ghostview
 
 set tkW3ConfigImageDirectView 1
 
+## Procedure to exec a command in a terminal
+
+proc tkW3ConfigExecTerm {command} {
+    global tkW3ConfigViewer
+    eval [concat "exec" $tkW3ConfigViewer(terminal) "-e" $command] 
+}
+
 ## The mail command
 proc tkW3ConfigSendMail {to subject body} {
     global env
-    if {$env(TK_WWW_MAIL) == "mhmail"} {
+    if [file executable /usr/ucb/mail] {
+	set file [open "| /usr/ucb/mail -s \"$subject\" $to" w+]
+    } elseif {$env(TK_WWW_MAIL) == "mhmail"} {
 	set file [open "| mhmail $to -subject \"$subject\"" w+]
-    } {
-	set file [open "| mail -s \"$subject\" $to" w+]
-    }
-
+    } 
     puts $file $body
     close $file
 }
@@ -78,7 +92,7 @@ set tkW3ConfigMenus {
 	    {DLG:set_entry_value .mail_dialog 2 [tkW3NavigateGetTitle];
 	     DLG:show . .mail_dialog} }
 	{}
-	{"Run TCL Command" {tkW3FileRunTclCommand} 0}
+	{"Run TCL Command" {tkW3FileRunTclCommand} 0 "<Meta-x>"}
 	{}
 	{"Close" tkW3FileQueryCloseWindow 0}
 	{"Exit" tkW3FileQueryExit 0}
@@ -114,6 +128,8 @@ set tkW3ConfigMenus {
 	{"Insert Glossary Item" {tkW3EditBeginGlossaryItem .f.msg} {} "<Meta-g>"}
 	{"Insert Image" {tkW3EditAddImage .f.msg} {} "<Meta-i>"}
 	{"Insert Paragraph Break" {tkW3EditInsertBullet .f.msg P} {} "<Meta-p>"}
+	{"Insert Line Break" {tkW3EditInsertBullet .f.msg BR} {}  "<Meta-b>"}
+	{"Insert Horizontal Rule" {tkW3EditInsertBullet .f.msg HR} {} "<Meta-h>"}
     }}
     {{paragraph "Paragraph" 0} {
 	{"Default" "tkW3EditSetStyle para {}"}
@@ -159,8 +175,8 @@ set tkW3ConfigMenus {
        {"Starting information sources" "tkW3HelpGetTopic sources" 0}
        {}
        {"Send mail to developer..." "DLG:show . .send_mail_dialog" 0}
-       {"Subscribe to mailing list tk-www@mit.edu" "tkW3HelpAddToMailingList" }
-       {"Unsubscribe to mailing list tk-www@mit.edu" "tkW3HelpRemoveFromMailingList" }
+       {"Subscribe to mailing list tkwww@vungtau.cerf.net" "tkW3HelpAddToMailingList" }
+       {"Unsubscribe to mailing list tkwww@vungtau.cerf.net" "tkW3HelpRemoveFromMailingList" }
 
        {}
        {"Using Help" 	 "tkW3HelpGetTopic using_help" 6}
@@ -211,7 +227,7 @@ proc tkW3ConfigDisplay {} {
     tkW3OutputToggleSet .toggles.is_modified $tkW3HtPage(modified)
 }
 
-set tkW3ConfigFileSuffixes   {
+set tkW3ConfigFileTypes   {
     { ".mime" "www/mime"  "8bit" 1.0 }	
     { ".PS" 	"application/postscript"  "8bit" 0.8 }
     { ".eps" 	"application/postscript"  "8bit" 0.8 }
@@ -226,15 +242,10 @@ set tkW3ConfigFileSuffixes   {
     { ".snd"  "audio/basic"  "binary" 1.0 }
     { ".au"   "audio/basic"  "binary" 1.0 }
     { ".bin" 	"application/octet-stream"  "binary" 1.0 }
-    { ".dmp.Z"  "image/compressed-x11-dump" "binary" 1.0}
     { ".dmp"  "image/x11-dump" "binary" 1.0}
-    { ".pbm.Z" "image/compressed-portable-bitmap" "binary" 1.0}
     { ".pbm" "image/portable-bitmap" "binary" 1.0}
-    { ".ppm.Z" "image/compressed-portable-pixmap" "binary" 1.0}
     { ".ppm" "image/portable-pixmap" "binary" 1.0}
     { ".xbm" "image/x-bitmap" "binary" 1.0} 
-    { ".xbm.Z" "image/compressed-xbm" "binary" 1.0}
-    { ".gif.Z" "image/compressed-gif" "binary" 1.0}
     { ".gif"  "image/gif"  "binary" 1.0 }
     { ".tiff" "image/x-tiff"  "binary" 1.0 }
     { ".jpg"  "image/jpeg"  "binary" 1.0 }
@@ -252,15 +263,17 @@ set tkW3ConfigFileSuffixes   {
     { ".dvi" "application/dvi" "binary" 1.0}
     { ".tar" "archive/tar" "binary" 1.0}
     { ".shar" "archive/shar" "binary" 1.0}
-    { ".Z" 	"application/x-compressed" "binary" 0.5 }
-    { ".gz"    "application/x-zipped" "binary" 0.5}
-    { ".x11.Z" "image/compressed-x11-dump" "binary" 1.0}
     { ".x11" "image/x11-dump" "binary" 1.0}
 }
 
-proc tkW3ConfigDisplayFile {type file_name address} {
+set tkW3ConfigFileEncodings {
+    { ".Z" "x-compressed" 1.0} 
+    { ".gz" "x-gzipped" 1.0}
+}
+
+proc tkW3ConfigDisplayFile {type file_name address {encoding ""}} {
     global tkW3ConfigViewer	
-    case $type {
+    switch -glob -- $type {
 	"image/*" {
 	    tkW3ImageDisplayFile $file_name
 	}
@@ -280,32 +293,40 @@ proc tkW3ConfigDisplayFile {type file_name address} {
 	tkW3NavigatePreface {TeX Device Independent file} "TeX DVI file" \
 	    "exec $tkW3ConfigViewer(dvi) $file_name &"
 	}
+	{application/tex} -
+	{application/latex} -
+	{application/octet-stream} -
 	{text/plain} {
-	    tkW3OutputSetBodyToFile $file_name
-	}
-	{application/x-compressed} {
-	    case [DLG:msg . .graphics_dialog "Compressed File" question \
- 		  "Display as text" "Display as image" "Save file as..."] {
-
-		      {1} {tkW3OutputSetBodyToFile "| zcat $file_name "}
-		      {2} {tkW3ImageDisplayFile $file_name}
-		      {3} {tkW3OutputSaveFileAs $file_name $address}
-		  } 
-	}
-	{application/x-zipped} {
-	    case [DLG:msg . .graphics_dialog "Gzip Compressed File" question \
-		  "Display as text" "Display as image" "Save file as..."] {
-
-		      {1} {tkW3OutputSetBodyToFile "| gunzip -c $file_name"}
-		      {2} {tkW3ImageDisplayFile $file_name}
-		      {3} {tkW3OutputSaveFileAs $file_name $address}
-		  }
+	    switch -- $encoding {
+		{x-compressed} {
+		    tkW3OutputSetBodyToFile "| zcat $file_name "
+		}
+		{x-gzipped} {
+		    tkW3OutputSetBodyToFile "| gunzip -c $file_name"
+		}
+		{default} {
+		    tkW3OutputSetBodyToFile $file_name
+		}
+	    }
 	}
 	{default} {
 	    case [DLG:msg . .graphics_dialog "Unknown file type" question \
 		  "Display as text" "Display as image" "Save file as..."] {
 
-		      {1} {tkW3OutputSetBodyToFile $file_name}
+		      {1} {
+			  case $encoding {
+			      {x-compressed} {
+				  tkW3OutputSetBodyToFile "| zcat $file_name "
+			      }
+			      {x-gzipped} {
+				  tkW3OutputSetBodyToFile \
+				      "| gunzip -c $file_name"
+			      }
+			      {default} {
+				  tkW3OutputSetBodyToFile $file_name
+			      }
+			  }
+		      }
 		      {2} {tkW3ImageDisplayFile $file_name}
 		      {3} {tkW3OutputSaveFileAs $file_name $address}
 		  }
